#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import doctest

sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))

import creole.html_emitter

if __name__ == "__main__":
    doctest.testmod(creole.html_emitter)



#!/usr/bin/python
# -*- coding: utf-8 -*-

class DocNode(object):
    """
    A node in the document.
    """

    def __init__(self, kind='', parent=None, content=None):
        self.children = []
        self.parent = parent
        self.kind = kind
        self.content = content
        if self.parent is not None:
            self.parent.children.append(self)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

ur"""
Test cases contributed by Jan Klopper (janklopper@underdark.nl),
modified by Radomir Dopieralski (MoinMoin:RadomirDopieralski).

>>> import lxml.html.usedoctest
>>> import creole
>>> def parse(text):
...     text = GenshiEmitter(creole.Parser(text).parse()).emit().render()
...     print unicode(text, 'utf-8')
>>> def wiki_parse(text):
...     rules = creole.Rules(wiki_words=True)
...     print GenshiEmitter(creole.Parser(text, rules).parse()).emit().render()

>>> parse(u'test')
<p>test</p>

>>> parse(u'test\ntest')
<p>test test</p>

>>> parse(u'test\n\ntest')
<p>test</p><p>test</p>

>>> parse(u'test\\\\test')
<p>test<br>test</p>

>>> parse(u'ÓÔÕÖØÙÚÛÜÝßàáâãäåæçèéêëìíîïñòóôõöøùúûüýÿŒœ%0A')
<p>ÓÔÕÖØÙÚÛÜÝßàáâãäåæçèéêëìíîïñòóôõöøùúûüýÿŒœ%0A</p>

>>> parse(u'----')
<hr>

>>> parse(u'==test==')
<h2>test</h2>

>>> parse(u'== test')
<h2>test</h2>

>>> parse(u'==test====')
<h2>test</h2>

>>> parse(u'=====test')
<h5>test</h5>

>>> parse(u'==test==\ntest\n===test===')
<h2>test</h2>
<p>test</p>
<h3>test</h3>

>>> parse(u'test\n* test line one\n * test line two\ntest\n\ntest')
<p>test</p>
<ul>
    <li>test line one</li>
    <li>test line two test</li>
</ul>
<p>test</p>

>>> parse(u'* test line one\n* test line two\n** Nested item')
<ul>
    <li>test line one</li>
    <li>test line two<ul>
        <li>Nested item</li>
    </ul></li>
</ul>

>>> parse(u'* test line one\n* test line two\n# Nested item')
<ul>
    <li>test line one</li>
    <li>test line two<ol>
        <li>Nested item</li>
    </ol></li>
</ul>

>>> parse(u'test //test test// test **test test** test')
<p>test <i>test test</i> test <b>test test</b> test</p>

>>> parse(u'test //test **test// test** test')
<p>test <i>test <b>test<i> test<b> test</b></i></b></i></p>

>>> parse(u'**test')
<p><b>test</b></p>

>>> parse(u'|x|y|z|\n|a|b|c|\n|d|e|f|\ntest')
<table>
    <tr><td>x</td><td>y</td><td>z</td></tr>
    <tr><td>a</td><td>b</td><td>c</td></tr>
    <tr><td>d</td><td>e</td><td>f</td></tr>
</table>
<p>test</p>

>>> parse(u'|=x|y|=z=|\n|a|b|c|\n|d|e|f|')
<table>
    <tr><th>x</th><td>y</td><th>z</th></tr>
    <tr><td>a</td><td>b</td><td>c</td></tr>
    <tr><td>d</td><td>e</td><td>f</td></tr>
</table>

>>> parse(u'test http://example.com/test test')
<p>test <a href="http://example.com/test" class="external">http://example.com/test</a> test</p>

>>> parse(u'http://example.com/,test, test')
<p><a href="http://example.com/,test" class="external">http://example.com/,test</a>, test</p>

>>> parse(u'(http://example.com/test)')
<p>(<a href="http://example.com/test" class="external">http://example.com/test</a>)</p>

XXX This might be considered a bug, but it's impossible to detect in general.
>>> parse(u'http://example.com/(test)')
<p><a href="http://example.com/(test" class="external">http://example.com/(test</a>)</p>

>>> parse(u'http://example.com/test?test&test=1')
<p><a href="http://example.com/test?test&amp;test=1" class="external">http://example.com/test?test&amp;test=1</a></p>

>>> parse(u'~http://example.com/test')
<p>http://example.com/test</p>

>>> parse(u'http://example.com/~test')
<p><a href="http://example.com/~test" class="external">http://example.com/~test</a></p>

>>> parse(u'[[test]] [[tset|test]]')
<p><a href="test" class="internal">test</a> <a href="tset" class="internal">test</a></p>

>>> parse(u'[[http://example.com|test]]')
<p><a href="http://example.com" class="external">test</a></p>

>>> wiki_parse(u'Lorem WikiWord iPsum sit ameT.')
<p>Lorem <a href="WikiWord" class="internal">WikiWord</a> iPsum sit ameT.</p>

"""


import re

from creole.parser import Parser
from creole.rules import LinkRules

from genshi import Stream, QName, Attrs

START, END, TEXT = Stream.START, Stream.END, Stream.TEXT

POS = (None, None, None)

class GenshiEmitter(object):
    """
    Generate Genshi stream output for the document
    tree consisting of DocNodes.
    """

    def __init__(self, root, link_rules=None):
        self.root = root
        self.link_rules = link_rules or LinkRules()

    def get_text(self, node):
        """Try to emit whatever text is in the node."""

        try:
            return node.children[0].content or ''
        except:
            return node.content or ''

    def wrap(self, tag, node):
        yield START, (QName(tag), Attrs()), POS
        for part in self.emit_children(node):
            yield part
        yield END, QName(tag), POS

    # *_emit methods for emitting nodes of the document:

    def document_emit(self, node):
        return self.emit_children(node)

    def text_emit(self, node):
        yield TEXT, node.content, POS

    def separator_emit(self, node):
        yield START, (QName('hr'), Attrs()), POS
        yield END, QName('hr'), POS

    def paragraph_emit(self, node):
        return self.wrap('p', node)

    def bullet_list_emit(self, node):
        return self.wrap('ul', node)

    def number_list_emit(self, node):
        return self.wrap('ol', node)

    def list_item_emit(self, node):
        return self.wrap('li', node)

    def table_emit(self, node):
        return self.wrap('table', node)

    def table_row_emit(self, node):
        return self.wrap('tr', node)

    def table_cell_emit(self, node):
        return self.wrap('td', node)

    def table_head_emit(self, node):
        return self.wrap('th', node)

    def emphasis_emit(self, node):
        return self.wrap('i', node)

    def strong_emit(self, node):
        return self.wrap('b', node)

    def header_emit(self, node):
        yield START, (QName('h%d' % node.level), Attrs()), POS
        yield TEXT, node.content, POS
        yield END, QName('h%d' % node.level), POS

    def code_emit(self, node):
        return self.wrap('tt', node)

    def link_emit(self, node):
        target = node.content
        class_ = "internal"
        m = self.link_rules.addr_re.match(target)
        if m:
            if m.group('extern_addr'):
                class_ = "external"
            elif m.group('inter_wiki'):
                raise NotImplementedError
        yield START, (QName('a'),
                      Attrs([
                            (QName('href'), target),
                            (QName('class'), class_),
                            ])), POS
        if node.children:
            for part in self.emit_children(node):
                yield part
        else:
            yield TEXT, target, POS
        yield END, QName('a'), POS

    def image_emit(self, node):
        target = node.content
        text = self.get_text(node)
        class_ = "internal"
        m = self.link_rules.addr_re.match(target)
        if m:
            if m.group('extern_addr'):
                class_ = "external"
            elif m.group('inter_wiki'):
                raise NotImplementedError
        yield START, (QName('img'),
                      Attrs([
                        (QName('src'), target),
                        (QName('alt'), text),
                        (QName('class'), class_),
                      ])), POS
        yield END, QName('img'), POS

    def macro_emit(self, node):
        raise NotImplementedError

    def break_emit(self, node):
        yield START, (QName('br'), Attrs()), POS
        yield END, QName('br'), POS

    def preformatted_emit(self, node):
        yield START, (QName('pre'), Attrs()), POS
        yield TEXT, node.content, POS
        yield END, QName('pre'), POS

    def default_emit(self, node):
        """Fallback function for emitting unknown nodes."""

        raise TypeError('Unknown node type')

    def emit_children(self, node):
        """Emit all the children of a node."""

        for child in node.children:
            for part in self.emit_node(child):
                yield part

    def emit_node(self, node):
        """Emit a single node."""

        emit = getattr(self, '%s_emit' % node.kind, self.default_emit)
        return emit(node)

    def emit(self):
        """Emit the document represented by self.root DOM tree."""

        return Stream(self.emit_node(self.root))





#!/usr/bin/python
# -*- coding: utf-8 -*-

import creole

class HtmlEmitter(creole.HtmlEmitter):
    """
    This class changes how we actually emit the HTML for some
    of the nodes. You should also override the link generation
    and macro support code here.
    """

    def header_emit(self, node):
        """Instead of using node.content, we emit the children."""

        return u'<h%d>%s</h%d>\n' % (node.level, self.emit_children(node),
                                     node.level)


class Rules(creole.Rules):
    """
    This class would be used to add or change any regular expressions.
    We don't need it in this particular case though, so we just leave
    it like the original class.
    """

class Parser(creole.Parser):
    r"""
    This class is for overriding the behavior of our parser. We change the
    rule that handles headings, so that instead of just putting the text
    inside the heading, it parses it, looking for any markup.

    Here are some simple tests:

    >>> import lxml.html.usedoctest
    >>> def parse(text):
    ...     print HtmlEmitter(Parser(text).parse()).emit()

    >>> parse(u'====test====')
    <h4>test</h4>

    >>> parse(u'===[[test]]===')
    <h3><a href="test">test</a></h3>

    >>> parse(u'==test //italic// **bold** test==')
    <h2>test <i>italic</i> <b>bold</b> test</h2>

    >>> parse(u'==test==\ntest')
    <h2>test</h2>
    <p>test</p>
    """

    def _head_repl(self, groups):
        # Note: this code can be shorter, I wrote it like this
        # so that I can comment on every step.
        #
        # The shorter version would look like this:
        #
        # self.cur = creole.DocNode('header', self.cur)
        # self.cur.level = len(groups.get('head_head', ' '))
        # self.parse_inline(groups.get('head_text', '').strip())
        # self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))
        # self.text = None

        # Make sure we are not inside a paragraph, list, other heading, etc.
        self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))
        # Remember where we are.
        parent = self.cur
        # Create the heading node.
        node = creole.DocNode('header', self.cur)
        # Set its level.
        node.level = len(groups.get('head_head', ' '))
        # Make it the curret node.
        self.cur = node
        # Parse the text inside the heading.
        text = groups.get('head_text', '').strip()
        self.parse_inline(text)
        # Go back to where we started.
        self.cur = parent
        self.text = None

if __name__=="__main__":
#    import sys
#    raw = unicode(sys.stdin.read(), 'utf-8', 'ignore')
#    rules = Rules()
#    document = Parser(raw, rules).parse()
#    html = HtmlEmitter(document).emit()
#    sys.stdout.write(html.encode('utf-8', 'ignore'))

# Instead of the above code to convert text from command line, we run tests:

    import doctest
    doctest.testmod()




#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys

from creole import Parser
from creole.html_emitter import HtmlEmitter

if __name__=="__main__":
    document = Parser(unicode(sys.stdin.read(), 'utf-8', 'ignore')).parse()
    sys.stdout.write(HtmlEmitter(document).emit().encode('utf-8', 'ignore'))




#!/usr/bin/python
# -*- coding: utf-8 -*-

import re
import sys


class LinkRules(object):
    """Rules for recognizing external links."""

    # For the link targets:
    proto = r'http|https|ftp|nntp|news|mailto|telnet|file|irc'
    extern = r'(?P<extern_addr>(?P<extern_proto>%s):.*)' % proto
    interwiki = r'''
            (?P<inter_wiki> [A-Z][a-zA-Z]+ ) :
            (?P<inter_page> .* )
        '''

    def __init__(self):
        self.addr_re = re.compile('|'.join([
                self.extern,
                self.interwiki,
            ]), re.X | re.U) # for addresses


class Rules(object):
    """Hold all the rules for generating regular expressions."""

    # For the inline elements:
    proto = r'http|https|ftp|nntp|news|mailto|telnet|file|irc'
    link = r'''(?P<link>
            \[\[
            (?P<link_target>.+?) \s*
            ([|] \s* (?P<link_text>.+?) \s*)?
            ]]
        )'''
    image = r'''(?P<image>
            {{
            (?P<image_target>.+?) \s*
            ([|] \s* (?P<image_text>.+?) \s*)?
            }}
        )'''
    macro = r'''(?P<macro>
            <<
            (?P<macro_name> \w+)
            (\( (?P<macro_args> .*?) \))? \s*
            ([|] \s* (?P<macro_text> .+?) \s* )?
            >>
        )'''
    code = r'(?P<code> {{{ (?P<code_text>.*?) }}} )'
    emph = r'(?P<emph> (?<!:)// )' # there must be no : in front of the //
                                   # avoids italic rendering in urls with
                                   # unknown protocols
    strong = r'(?P<strong> \*\* )'
    linebreak = r'(?P<break> \\\\ )'
    escape = r'(?P<escape> ~ (?P<escaped_char>\S) )'
    char =  r'(?P<char> . )'

    # For the block elements:
    separator = r'(?P<separator> ^ \s* ---- \s* $ )' # horizontal line
    line = r'(?P<line> ^ \s* $ )' # empty line that separates paragraphs
    head = r'''(?P<head>
            ^ \s*
            (?P<head_head>=+) \s*
            (?P<head_text> .*? ) \s*
            (?P<head_tail>=*) \s*
            $
        )'''
    text = r'(?P<text> .+ )'
    list = r'''(?P<list>
            ^ [ \t]* ([*][^*\#]|[\#][^\#*]).* $
            ( \n[ \t]* [*\#]+.* $ )*
        )''' # Matches the whole list, separate items are parsed later. The
             # list *must* start with a single bullet.
    item = r'''(?P<item>
            ^ \s*
            (?P<item_head> [\#*]+) \s*
            (?P<item_text> .*?)
            $
        )''' # Matches single list items
    pre = r'''(?P<pre>
            ^{{{ \s* $
            (\n)?
            (?P<pre_text>
                ([\#]!(?P<pre_kind>\w*?)(\s+.*)?$)?
                (.|\n)+?
            )
            (\n)?
            ^}}} \s*$
        )'''
    pre_escape = r' ^(?P<indent>\s*) ~ (?P<rest> \}\}\} \s*) $'
    table = r'''(?P<table>
            ^ \s*
            [|].*? \s*
            [|]? \s*
            $
        )'''

    # For splitting table cells:
    cell = r'''
            \| \s*
            (
                (?P<head> [=][^|]+ ) |
                (?P<cell> (  %s | [^|])+ )
            ) \s*
        ''' % '|'.join([link, macro, image, code])

    def __init__(self, bloglike_lines=False, url_protocols=None,
                 wiki_words=False):
        c = re.compile
        # For pre escaping, in creole 1.0 done with ~:
        self.pre_escape_re = c(self.pre_escape, re.M | re.X)
        # for link descriptions
        self.link_re = c('|'.join([self.image, self.linebreak,
                                   self.char]), re.X | re.U)
        # for list items
        self.item_re = c(self.item, re.X | re.U | re.M)
        # for table cells
        self.cell_re = c(self.cell, re.X | re.U)

        # For block elements:
        if bloglike_lines:
            self.text = r'(?P<text> .+ ) (?P<break> (?<!\\)$\n(?!\s*$) )?'
        self.block_re = c('|'.join([self.line, self.head, self.separator,
                                    self.pre, self.list, self.table,
                                    self.text]), re.X | re.U | re.M)

        # For inline elements:
        if url_protocols is not None:
            self.proto = '|'.join(re.escape(p) for p in url_protocols)
        self.url =  r'''(?P<url>
            (^ | (?<=\s | [.,:;!?()/=]))
            (?P<escaped_url>~)?
            (?P<url_target> (?P<url_proto> %s ):\S+? )
            ($ | (?=\s | [,.:;!?()] (\s | $))))''' % self.proto
        inline_elements = [self.link, self.url, self.macro,
                           self.code, self.image, self.strong,
                           self.emph, self.linebreak,
                           self.escape, self.char]
        if wiki_words:
            import unicodedata
            up_case = u''.join(unichr(i) for i in xrange(sys.maxunicode)
                               if unicodedata.category(unichr(i))=='Lu')
            self.wiki = ur'''(?P<wiki>[%s]\w+[%s]\w+)''' % (up_case, up_case)
            inline_elements.insert(3, self.wiki)
        self.inline_re = c('|'.join(inline_elements), re.X | re.U)





#!/usr/bin/python
# -*- coding: utf-8 -*-

import creole

config = dict(
    name='creole',
    version=creole.__version__,
    url='http://oink.sheep.art.pl/WikiCreole%20parser%20in%20python',
    download_url=('http://oink.sheep.art.pl/+download/creole-%s.tar.gz'
                  % creole.__version__),
    license='GNU General Public License (GPL), BSD',
    author='Radomir Dopieralski, Thomas Waldmann',
    author_email='creole@sheep.art.pl',
    description='Parser for WikiCreole text markup.',
    long_description=creole.__doc__,
    keywords='wiki wikicreole creole markup text',
    packages=['creole'],
    data_files=[
        ('share/doc/creole.py/examples',
         ['examples/creole2html.py', 'examples/headings.py']),
        ('share/doc/creole.py/tests', ['tests/test_html_emitter.py']),
        ('share/doc/creole.py', ['COPYING']),
    ],
    scripts=['examples/creole2html.py'],
    platforms='any',
    classifiers=[
        'Topic :: Text Processing :: Markup',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Intended Audience :: Developers',
        'Topic :: Communications',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
    ],
)

from distutils.core import setup
setup(**config)




#!/usr/bin/python
# -*- coding: utf-8 -*-

import re
import sys

from creole.rules import Rules
from creole.document import DocNode


class Parser(object):
    """
    Parse the raw text and create a document object
    that can be converted into output using Emitter.

    A separate instance should be created for parsing a new document.
    The first parameter is the raw text to be parsed. An optional second
    argument is the Rules object to use. You can customize the parsing
    rules to enable optional features or extend the parser.
    """

    def __init__(self, raw, rules=None):
        self.rules = rules or Rules()
        self.raw = raw
        self.root = DocNode('document', None)
        self.cur = self.root        # The most recent document node
        self.text = None            # The node to add inline characters to

    def _upto(self, node, kinds):
        """
        Look up the tree to the first occurence
        of one of the listed kinds of nodes or root.
        Start at the node node.
        """
        while node.parent is not None and not node.kind in kinds:
            node = node.parent
        return node

    # The _*_repl methods called for matches in regexps. Sometimes the
    # same method needs several names, because of group names in regexps.

    def _url_repl(self, groups):
        """Handle raw urls in text."""

        if not groups.get('escaped_url'):
            # this url is NOT escaped
            target = groups.get('url_target', '')
            node = DocNode('link', self.cur)
            node.content = target
            DocNode('text', node, node.content)
            self.text = None
        else:
            # this url is escaped, we render it as text
            if self.text is None:
                self.text = DocNode('text', self.cur, u'')
            self.text.content += groups.get('url_target')

    def _link_repl(self, groups):
        """Handle all kinds of links."""

        target = groups.get('link_target', '')
        text = (groups.get('link_text', '') or '').strip()
        parent = self.cur
        self.cur = DocNode('link', self.cur)
        self.cur.content = target
        self.text = None
        self.parse_re(text, self.rules.link_re)
        self.cur = parent
        self.text = None

    def _wiki_repl(self, groups):
        """Handle WikiWord links, if enabled."""

        text = groups.get('wiki', '')
        node = DocNode('link', self.cur)
        node.content = text
        DocNode('text', node, node.content)
        self.text = None

    def _macro_repl(self, groups):
        """Handles macros using the placeholder syntax."""

        name = groups.get('macro_name', '')
        text = (groups.get('macro_text', '') or '').strip()
        node = DocNode('macro', self.cur, name)
        node.args = groups.get('macro_args', '') or ''
        DocNode('text', node, text or name)
        self.text = None

    def _image_repl(self, groups):
        """Handles images and attachemnts included in the page."""

        target = groups.get('image_target', '').strip()
        text = (groups.get('image_text', '') or '').strip()
        node = DocNode("image", self.cur, target)
        DocNode('text', node, text or node.content)
        self.text = None

    def _separator_repl(self, groups):
        self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))
        DocNode('separator', self.cur)

    def _item_repl(self, groups):
        bullet = groups.get('item_head', u'')
        text = groups.get('item_text', u'')
        if bullet[-1] == '#':
            kind = 'number_list'
        else:
            kind = 'bullet_list'
        level = len(bullet)
        lst = self.cur
        # Find a list of the same kind and level up the tree
        while (lst and
                   not (lst.kind in ('number_list', 'bullet_list') and
                        lst.level == level) and
                    not lst.kind in ('document', 'section', 'blockquote')):
            lst = lst.parent
        if lst and lst.kind == kind:
            self.cur = lst
        else:
            # Create a new level of list
            self.cur = self._upto(self.cur,
                ('list_item', 'document', 'section', 'blockquote'))
            self.cur = DocNode(kind, self.cur)
            self.cur.level = level
        self.cur = DocNode('list_item', self.cur)
        self.parse_inline(text)
        self.text = None

    def _list_repl(self, groups):
        text = groups.get('list', u'')
        self.parse_re(text, self.rules.item_re)

    def _head_repl(self, groups):
        self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))
        node = DocNode('header', self.cur, groups.get('head_text', '').strip())
        node.level = len(groups.get('head_head', ' '))

    def _text_repl(self, groups):
        text = groups.get('text', '')
        if self.cur.kind in ('table', 'table_row', 'bullet_list',
            'number_list'):
            self.cur = self._upto(self.cur,
                ('document', 'section', 'blockquote'))
        if self.cur.kind in ('document', 'section', 'blockquote'):
            self.cur = DocNode('paragraph', self.cur)
        else:
            text = u' ' + text
        self.parse_inline(text)
        if groups.get('break') and self.cur.kind in ('paragraph',
            'emphasis', 'strong', 'code'):
            DocNode('break', self.cur, '')
        self.text = None
    _break_repl = _text_repl

    def _table_repl(self, groups):
        row = groups.get('table', '|').strip()
        self.cur = self._upto(self.cur, (
            'table', 'document', 'section', 'blockquote'))
        if self.cur.kind != 'table':
            self.cur = DocNode('table', self.cur)
        tb = self.cur
        tr = DocNode('table_row', tb)

        text = ''
        for m in self.rules.cell_re.finditer(row):
            cell = m.group('cell')
            if cell:
                self.cur = DocNode('table_cell', tr)
                self.text = None
                self.parse_inline(cell)
            else:
                cell = m.group('head')
                self.cur = DocNode('table_head', tr)
                self.text = DocNode('text', self.cur, u'')
                self.text.content = cell.strip('=')
        self.cur = tb
        self.text = None

    def _pre_repl(self, groups):
        self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))
        kind = groups.get('pre_kind', None)
        text = groups.get('pre_text', u'')
        def remove_tilde(m):
            return m.group('indent') + m.group('rest')
        text = self.rules.pre_escape_re.sub(remove_tilde, text)
        node = DocNode('preformatted', self.cur, text)
        node.sect = kind or ''
        self.text = None

    def _line_repl(self, groups):
        self.cur = self._upto(self.cur, ('document', 'section', 'blockquote'))

    def _code_repl(self, groups):
        DocNode('code', self.cur, groups.get('code_text', u'').strip())
        self.text = None

    def _emph_repl(self, groups):
        if self.cur.kind != 'emphasis':
            self.cur = DocNode('emphasis', self.cur)
        else:
            self.cur = self._upto(self.cur, ('emphasis', )).parent
        self.text = None

    def _strong_repl(self, groups):
        if self.cur.kind != 'strong':
            self.cur = DocNode('strong', self.cur)
        else:
            self.cur = self._upto(self.cur, ('strong', )).parent
        self.text = None

    def _break_repl(self, groups):
        DocNode('break', self.cur, None)
        self.text = None

    def _escape_repl(self, groups):
        if self.text is None:
            self.text = DocNode('text', self.cur, u'')
        self.text.content += groups.get('escaped_char', u'')

    def _char_repl(self, groups):
        if self.text is None:
            self.text = DocNode('text', self.cur, u'')
        self.text.content += groups.get('char', u'')

    def parse_inline(self, raw):
        """Recognize inline elements inside blocks."""

        self.parse_re(raw, self.rules.inline_re)

    def parse_re(self, raw, rules_re):
        """Parse a fragment according to the compiled rules."""

        for match in rules_re.finditer(raw):
            groups = dict((k, v) for (k, v)
                          in match.groupdict().iteritems()
                          if v is not None)
            name = match.lastgroup
            function = getattr(self, '_%s_repl' % name)
            function(groups)

    def parse(self):
        """Parse the text given as self.raw and return DOM tree."""

        self.parse_re(self.raw, self.rules.block_re)
        return self.root




#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
    Creole wiki markup parser

    See http://wikicreole.org/ for latest specs.

    Notes:
    * No markup allowed in headings.
      Creole 1.0 does not require us to support this.
    * No markup allowed in table headings.
      Creole 1.0 does not require us to support this.
    * No (non-bracketed) generic url recognition: this is "mission impossible"
      except if you want to risk lots of false positives. Only known protocols
      are recognized.
    * We do not allow ":" before "//" italic markup to avoid urls with
      unrecognized schemes (like wtf://server/path) triggering italic rendering
      for the rest of the paragraph.

    @copyright: 2007 MoinMoin:RadomirDopieralski (creole 0.5 implementation),
                2007 MoinMoin:ThomasWaldmann (updates)
    @license: GNU GPL, see COPYING for details.
    @license: BSD, see COPYING for details.
"""

__version__ = '1.2'

from creole.rules import Rules
from creole.parser import Parser
from creole.document import DocNode

